/* Copyright (C) 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import java.util.List;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.model.minfo.BedHeightSingle;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.model.BedHeightSingleValue;
import org.dive4elements.river.themes.ThemeDocument;

import org.dive4elements.river.artifacts.model.minfo.BedDiameterData;
import org.dive4elements.river.artifacts.model.minfo.BedDiameterResult;
import org.dive4elements.river.artifacts.model.minfo.BedloadDiameterResult;

public class BedQualityDiameterProcessor extends DefaultProcessor {

    private final static Logger logger =
            Logger.getLogger(BedQualityDiameterProcessor.class);

    public static final String I18N_AXIS_LABEL_DEFAULT =
        "Durchmesser [mm]";
    public static final String I18N_AXIS_LABEL =
        "chart.bedquality.yaxis.label.diameter";

    @Override
    public void doOut(
            DiagramGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument    theme,
            boolean          visible) {
        CallContext context = generator.getCallContext();
        XYSeries series = new StyledXYSeries(bundle.getFacetDescription(),
                theme);
        Object data = bundle.getData(context);
        String facetName = bundle.getFacetName();
        double [][] points;

        if (facetName.equals(FacetTypes.BED_QUALITY_BED_DIAMETER_TOPLAYER)) {
            points = ((BedDiameterResult) data).getDiameterCapData();
        } else if (facetName.equals(FacetTypes.BED_QUALITY_BED_DIAMETER_SUBLAYER)) {
            points = ((BedDiameterResult) data).getDiameterSubData();
        } else if (facetName.equals(FacetTypes.BED_QUALITY_BEDLOAD_DIAMETER)) {
            points = ((BedloadDiameterResult) data).getDiameterData();
        } else if (facetName.equals(FacetTypes.BED_DIAMETER_DATA_TOP) ||
                 facetName.equals(FacetTypes.BED_DIAMETER_DATA_SUB) ||
                 facetName.equals(FacetTypes.BEDLOAD_DIAMETER_DATA)) {
            points = ((BedDiameterData) data).getDiameterData();
        } else {
            logger.error("Unknown facet name: " + facetName);
            return;
        }
        StyledSeriesBuilder.addPoints(series, points, true);

        generator.addAxisSeries(series, axisName, visible);
    }

    @Override
    public boolean canHandle(String facettype) {
        return facettype.equals(FacetTypes.BED_QUALITY_BED_DIAMETER_TOPLAYER) ||
            facettype.equals(FacetTypes.BED_QUALITY_BED_DIAMETER_SUBLAYER) ||
            facettype.equals(FacetTypes.BED_QUALITY_BEDLOAD_DIAMETER) ||
            facettype.equals(FacetTypes.BED_DIAMETER_DATA_TOP) ||
            facettype.equals(FacetTypes.BED_DIAMETER_DATA_SUB) ||
            facettype.equals(FacetTypes.BEDLOAD_DIAMETER_DATA);
    }

    @Override
    public String getAxisLabel(DiagramGenerator generator) {
        return generator.msg(
                I18N_AXIS_LABEL,
                I18N_AXIS_LABEL_DEFAULT);
    }
}

